import typer
from typing import Optional
from rich import print

from pixelfuse.src.f2v import ToVideo
from pixelfuse.src.v2f import ToFile
from pixelfuse.src.old_files.d_v1 import ToFile as dv1ToFile

app = typer.Typer()

@app.command(name="videoToFile")
def convertToFile(path: str, verbose: Optional[int]=2, decoder: Optional[str]=""):
    try:
        match decoder:
            case "":
                c = ToFile(path, verbose)
                c.convert()
            case "v1":
                c = dv1ToFile(path, verbose)
                c.convert()
    except FileExistsError as e:
        print(f"[red bold]{e}")
    except UnicodeDecodeError as e:
        print(f"[red bold]Cannot decode frames:[/red bold] {e}")
    except Exception as e:
        print(f"[red bold]Unknown error:")
        print(e)

@app.command(name="fileToVideo")
def convertToVideo(
    path: str,
    fps: Optional[float]=1.,
    width: Optional[int]=640,
    height: Optional[int]=480,
    fourcc: Optional[str]="HFYU",
    output: Optional[str]="output.avi",
    verbose: Optional[int]=2,
):
    try:
        c = ToVideo(path, fps, width, height, fourcc, output, verbose)
        c.convert()
    except FileNotFoundError:
        print(f"[red bold]File {path} doesn't exist")
    except Exception as e:
        print("[red bold]Unknown error:")
        print(e)


import ffmpegio
import numpy as np
import hashlib
import pickle
from pixelfuse import decoder, compatible_encoders
from rich.console import Console

class ToFile:
    def __init__(self, filename, verbose, show_log):
        self.filename = filename
        self.verbose = verbose
        self.show_log = show_log
        self.console = Console()

    def read_video(self):
        self.print_verbose("Reading video...", 1)
        _, I = ffmpegio.video.read(self.filename, show_log=self.show_log)
        self.I = np.flip(I, axis=3)
        del I

    def load_meta(self):
        self.print_verbose("Loading meta...", 2)
        self.meta = pickle.loads(bytes(self.I[-1].flatten()))

        if self.meta["encoder"] not in compatible_encoders:
            self.print_verbose(f"[yellow bold]Endcoder {self.meta['encoder']} isn't compatible with {decoder}", 0)

    def preprocess_file_data(self):
        self.print_verbose("Preprocessing file data...", 2)
        data = self.I[:-1].flatten()
        del self.I
        data = data.reshape((-1, 3))
        while np.all(data[-1] == 0):
            data = data[:-1]
        data = data.flatten()
        self.data = data[:len(data)-self.meta["lastZeros"]]
        del data

    def write_file(self):
        self.print_verbose("Writing file data...", 1)
        with open(self.meta["filename"], "wb") as f:
            f.write(bytes(self.data))
        del self.data

    def verify(self):
        self.print_verbose("Verifing file data...", 2)
        if self.meta["checksum"] != hashlib.sha512(open(self.meta["filename"], 'rb').read()).hexdigest():
            self.print_verbose("[red bold]File checksum doesn't match!", np.NINF)
        del self.meta

    def convert(self):
        with self.console.status("[green]Decoding video...") as s:
            self.s = s 
            self.read_video()
            self.load_meta()
            # self.preprocess_file_data()
            # self.write_file()
            # self.verify()

    def print_verbose(self, text, verbose):
        if self.verbose >= verbose:
            self.console.print(text)


import cv2 
import pickle
import numpy as np
import tempfile
import os
import hashlib
from rich.console import Console
from pixelfuse import decoder, compatible_encoders

class ToFile:
    def __init__(self, filename, verbose):
        self.cap = cv2.VideoCapture(filename)
        self.console = Console()
        self.verbose = verbose

        if not self.cap.isOpened(): 
            raise FileNotFoundError("Error opening video file")

    def writeTempFile(self):
        self.print("Start reading video", 1)
        ret, frame = self.cap.read()
        toWriteFrame = None
        self.meta = None
        self.temp = tempfile.NamedTemporaryFile(mode="wb", delete=False, dir="./")
        self.print(f"Created temp file {self.temp.name}", 2)

        while(ret):
            if all(frame[0, 0] == [128, 4, 149]):
                self.print("Found [128  4 149] meta frame", 1)
                self.meta = pickle.loads(bytes(frame.flatten()))

                if self.meta["encoder"] not in compatible_encoders:
                    self.print_verbose(f"[yellow bold]Endcoder {self.meta['encoder']} isn't compatible with {decoder}", 0)

            if toWriteFrame is not None and self.meta is not None:
                self.print(f"Preprocessing last {toWriteFrame[0, 0]} frame...", 1)

                toWriteFrame = toWriteFrame.flatten().reshape((-1, 3))
                while np.all(toWriteFrame[-1] == 0):
                    toWriteFrame = toWriteFrame[:-1]
                toWriteFrame = toWriteFrame.flatten()
                toWriteFrame = toWriteFrame[:len(toWriteFrame)-self.meta["lastZeros"]]

                self.temp.write(toWriteFrame)
                break

            if toWriteFrame is not None:
                self.temp.write(toWriteFrame)

            self.print(f"[green]Done [/green]{frame[0, 0]} [green]frame[/green]", 2)
            
            toWriteFrame = frame
            ret, frame = self.cap.read()

        self.temp.close()
        self.print("[green bold]Done writing temp file", 1)

    def renameAccordingToMeta(self):
        try:
            self.print(f"Rename temp file: {self.temp.name} -> ./{self.meta['filename']}", 1)
            os.link(self.temp.name, self.meta["filename"])
        except FileExistsError:
            os.remove(self.temp.name)
            raise FileExistsError(f"Cannot write file, file already exists: {self.meta['filename']}")

        self.print(f"Remove temp file {self.temp.name}", 2)
        os.remove(self.temp.name)

    def verify(self):
        self.print(f"Verify file", 2)
        if self.meta["checksum"] != hashlib.sha512(open(self.meta["filename"], 'rb').read()).hexdigest():
            print("Doesn't match")

    def convert(self):
        with self.console.status("[green]Decoding video..."):
            self.writeTempFile()
            self.renameAccordingToMeta()
            self.verify()

    def print(self, text, verbose):
        if self.verbose >= verbose:
            self.console.print(text)